<?php

namespace Drupal\commerce_paybox_payment\Access;

use Drupal\commerce_paybox_payment\SignatureChecker;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Custom Access check service.
 */
class PaymentReturnAccessCheck implements AccessInterface {

  /**
   * Request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  private $request;

  /**
   * Commerce Payment Storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $commercePaymentStorage;

  /**
   * PaymentReturnAccessCheck constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request Stack Service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManager Service.
   */
  public function __construct(RequestStack $requestStack, EntityTypeManagerInterface $entityTypeManager) {
    $this->request = $requestStack->getCurrentRequest();
    $this->commercePaymentStorage = $entityTypeManager->getStorage('commerce_payment');
  }

  /**
   * Manage access payment return.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current User.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Result Access.
   */
  public function access(AccountInterface $account) {
    if ($account->isAuthenticated()) {
      return AccessResult::forbidden();
    }

    foreach (['Ref', 'Mt', 'Signature', 'Error'] as $param) {
      if (empty($this->request->query->get($param))) {
        return AccessResult::forbidden();
      }
    }
    $payment = $this->commercePaymentStorage->load($this->request->query->get('Ref'));
    if (!$payment instanceof PaymentInterface) {
      return AccessResult::forbidden();
    }

    if ('completed' === $payment->getState()->getString()) {
      return AccessResult::forbidden();
    }

    if (1 !== SignatureChecker::checkSignature($this->request)) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

}
