<?php

namespace Drupal\commerce_paybox_payment\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Manage Paybox Server Calls.
 */
class PaymentController extends ControllerBase {

  const EMPTY_ERROR = '00000';
  const PENDING_CODE = '99999';

  /**
   * The Commerce Payment Storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $commercePaymentStorage;

  /**
   * PaymentController constructor.
   */
  public function __construct() {
    $this->entityTypeManager = $this->entityTypeManager();
    $this->commercePaymentStorage = $this->entityTypeManager->getStorage('commerce_payment');
  }

  /**
   * Method called by Paybox Server on payment return.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Basic Json Response.
   */
  public function return(Request $request) {
    $paymentId = $request->query->get('Ref');
    $returnAmount = $request->query->get('Mt');
    $transactionId = $request->query->get('Transaction');
    $transactionDate = $request->query->get('Date');
    $error = $request->query->get('Error');
    /** @var \Drupal\commerce_payment\Entity\Payment $payment */
    $payment = $this->commercePaymentStorage->load($paymentId);

    $paymentAmount = (float) $payment->getAmount()->getNumber() * 100;
    $paymentAmount = str_pad((string) $paymentAmount, 3, '0', STR_PAD_LEFT);
    if (($returnAmount !== $paymentAmount) || (!empty($error) && self::EMPTY_ERROR !== $error  && self::PENDING_CODE !== $error)) {

      if (self::EMPTY_ERROR === $error) {
        $error = 'Charged amount not equal to order amount';
      }
      $this->getLogger('commerce_paybox_payment')
        ->error('Error during api return paybox on payment :payment error :error', [
          'payment' => $payment->id(),
          'error' => $error,
        ]);

      $payment->setState('failure');
      $payment->set('remote_state', $error);
      $payment->save();
      return new JsonResponse(FALSE);
    }

    if (self::PENDING_CODE === $error) {
      $payment->setState('pending');
      $payment->save();

      return new JsonResponse(TRUE);
    }

    $payment->setState('completed');
    $payment->set('remote_id', $transactionId);
    $payment->set('authorized', strtotime('today ' . $transactionDate));
    $payment->save();
    $order = $payment->getOrder();
    $order->set('state', 'completed');
    $order->save();

    return new JsonResponse(TRUE);
  }

}
